// Номер телефона хозяина охраняемого объекта в международном формате
String phoneNumber = "+380678068008";

// Библиотека для общения с SIM800L 
#include <SoftwareSerial.h>
// Экземплярар класса для работы с GSM-модулем (RX=10  TX=9)
SoftwareSerial SIM800L =  SoftwareSerial(10, 9);

 // Флаг детектирования вибрации
volatile bool flagVibration = false;
// Текущая фаза или состояние программы
uint8_t globalState = 0;

// *** ФУНКЦИЯ ПРЕДВАРИТЕЛЬНЫХ УСТАНОВОК ***
void setup() {
  // Инициализация терминала и порта общения с GSM-модулем
  Serial.begin(9600);
  SIM800L.begin(9600);
  
  // Активация внешнего прерывания на входе D2 по смене лог. уровня
  attachInterrupt(0, detectFunc, CHANGE);
}

// *** ОСНОВНОЙ ЦИКЛ ***
void loop() {
  if(globalState == 0) {
    if(flagVibration) { // Если зафиксирован наклон или вибрация
      // Выводим в терминал сообщение о проникновении
      Serial.println("ALARM!!!");
      // Зовним абоненту
      callOut(phoneNumber);
    }
  }
  else if(globalState == 1) {
    // Разрешаем опрашивать датчик в прерывании
    flagVibration = true;
    globalState = 0;
  }
}

// --- ОБРАБОТЧИК ПРЕРЫВАНИЯ ПРО ИЗМЕНЕНИЮ ЛОГ. УРОВНЯ НА ВХОДЕ D2 ---
void detectFunc() {
  // Если есть наклон или вибрация, временно запрещаем опрос датчика
  // чтобы оповестить хозяина
  if(!flagVibration) flagVibration = true;
}

// --- ФУНКЦИЯ ПОСЫЛКИ КОМАНДЫ GSM-МОДУЛЮ SIM800L С ОЖИДАНИЕМ ОТВЕТА ---
String sendATCommand(String cmd) {
  uint32_t sacTimeout = millis() + 2000;
  String answer = "";
  
  Serial.print("Arduino->SIM800L: "); Serial.println(cmd);
  SIM800L.println(cmd);
  
  while(!SIM800L.available() && millis() < sacTimeout) {};
  
  if(SIM800L.available()) answer = SIM800L.readString();
  else answer = "timeout...";
  Serial.println("SIM800L->Arduino: "); Serial.println(answer);
  return answer;
}

// --- ФУНКЦИЯ ДОЗВОНА АБОНЕНТУ ---
void callOut(String number) {
  static uint32_t callOutTimeout = 0;
  static uint8_t callOutState = 0;

  if(callOutState == 0) {
    String a = sendATCommand("ATD" + number + ";"); // Набираем абонента
    a.trim();
    if(a.startsWith("OK")) {  // Если набор номера прошёл успешно
      callOutTimeout = millis() + 20000;  // Звонить будем в течение 20 секунд
      callOutState = 1; // Переходим к следующему этапу
    }
    else delay(200);  // Если набор номера прошёл неуспешно - повторяем процедуру через 200мс
  }
  else if(callOutState == 1) {
    if(millis() > callOutTimeout) { callOutState = 2; }
  }
  else if(callOutState == 2) { // В случае недозвона, выводим сообщение в терминал
    Serial.println("Call out timeout...");
    sendATCommand("ATH"); // Ложим трубку
    callOutState = 0;
    globalState++; // Переходим к следующей фазе
  }
}
